/***************************************************************************
                          catalogentry.h  -  description
                             -------------------
    begin                : Thu Mar 20 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CATALOGENTRY_H
#define CATALOGENTRY_H

#include <qstring.h>

class QStringList;
class QDate;

/**This class stores infos about an entry in the catalogue and provides basic methods to create or acces it
  *@author Stefano Salvador
  */

class CatalogEntry
{
public: 

  //* Public constructor that creates an entry
  CatalogEntry( QString n = "",
                int size = 0,
                QString d = "",
                QDate* = 0,
                QStringList* = 0,
                int entry_type = 0 );

  //* Enumerator used to identify the type of entry
  enum
  {
    E_LINK    = 1,
    E_DIR     = 2,
    E_FILE    = 4,
    E_CATALOG = 8
  };

  //* Public destructor
  ~CatalogEntry();

  //* Returns the name of the catalog
  QString MyCatalog() const;

  //* Returns the name of the entry
  QString Name() const;

  //* Returns the size of the entry
  int Size() const;

  //* Returns the description of the entry
  QString Description() const;

  //* Returns the date of the entry
  QDate* Date() const;

  //* Returns the Categories of the entry
  QStringList* CatList() const;

  //* Returns the type of the entry
  int EntryType() const;

  //* Returns the icon of the entry
  QString Icon() const;

  //* Set the name of the entry
  void SetName( QString name );

  //* Set the size of the entry
  void SetSize( int size );

  //* Set the description of the entry
  void SetDescription( QString description );

  //* Set the date of the entry
  void SetDate( QDate* date );

  //* Set the categories of the entry
  void SetCatList( QStringList* cl );

  //* Set the type of the entry
  void SetEntryType( int entry_type );

  //* Set the mime type of the entry
  void SetIcon( QString icon );

  //* Add a category to the entry
  void addCat( QString str );

  //* Removes a category from the list of categories
  void delCat( QString );

  //* Set the name of parent catalog
  void setMyCatalog( QString str );

private:
  //* The catalogue name
  QString myCatalog;

  //* The name of the entry
  QString name;

  //* The description of the entry
  QString description;

  //* The size of the entry
  int size;

  //* A date identifier of the entry
  QDate* date;

  //* List of category
  QStringList* cl;

  //* Icon
  QString icon;

  //* Type of entry
  int entry_type;
};

#endif
