/***************************************************************************
                          catalogview.cpp  -  description
                             -------------------
    begin                : Fri Mar 28 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qpainter.h>
#include <qsplitter.h>
#include <qdatetime.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qstringlist.h>

// include files for KDE
#include <klocale.h>
#include <klistview.h>
#include <kprogress.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kdirselectdialog.h>

// application specific includes
#include "katalogdoc.h"
#include "cataloghandler.h"
#include "catalogentry.h"
#include "catalogview.h"
#include "categoryhandler.h"
#include "cattreeitem.h"
#include "midview.h"
#include "eklistviewitem.h"

CatalogView::CatalogView( QWidget *parent, const char *name, KatalogDoc *pDoc ):
QWidget(parent,name)
{
  // Create the splitter of the main view
  QSplitter *split = new QSplitter( parent );

  // Pointer to the document
  doc = pDoc;

  treeItemP = 0;
  treeItemC = 0;
  lvMI = 0;

  // Left
  QVBox *left = new QVBox( split );
  // Left::Top
  lvM = new KListView( left );
  lvM->addColumn( i18n( "Catalogs" ) );
  lvM->setFullWidth();
  lvM->setRootIsDecorated( TRUE );

  // Left::Middle
  lmid = new MidView( left );
  lmid->hide();

  // Left::Bottom
  QHBox *lb = new QHBox( left );
  ladd = new QPushButton( i18n("Add"), lb );
  ledit = new QPushButton( i18n("Edit"), lb );
  QPushButton *ldel = new QPushButton( i18n("Remove"), lb );
  QLabel *dummy = new QLabel( "", lb );
  ladd->setFixedHeight( fontMetrics().height() + 2 );
  ledit->setFixedHeight( fontMetrics().height() + 2 );
  ldel->setFixedHeight( fontMetrics().height() + 2 );
  lb->setMinimumHeight( ladd->height() + 10 );
  ladd->setToggleButton( true );
  ledit->setToggleButton( true );
  lb->setSpacing( 6 );
  lb->setStretchFactor( dummy, 1 );

  // Right
  QVBox *right = new QVBox( split );

  // Right::Top
  lvD = new KListView( right );
  lvD->addColumn( i18n( "Filename" ) );
  lvD->addColumn( i18n( "Size" ) );
  lvD->addColumn( i18n( "Date" ) );
  lvD->addColumn( i18n( "Category" ) );
  lvD->addColumn( i18n( "Description" ) );
  lvD->setFullWidth();
  lvD->setSorting( -1 );

  // Right::Middle
  rmid = new MidView( right );
  rmid->MountEnabled( false );
  rmid->hide();

  // Right::Bottom
  QHBox *rb = new QHBox( right );
  rb->setMargin( 6 );
  redit = new QPushButton( i18n("Edit"), rb );
  QPushButton *rdel = new QPushButton( i18n("Remove"), rb );
  QLabel *dummyr = new QLabel( "", rb );
  redit->setFixedHeight( fontMetrics().height() + 2 );
  rdel->setFixedHeight( fontMetrics().height() + 2 );
  rb->setMinimumHeight( redit->height() + 10 );
  redit->setToggleButton( true );
  rb->setSpacing( 6 );
  rb->setStretchFactor( dummyr, 1 );

  // Adjust sizes
  QValueList<int> vl;
  vl.append(80);
  vl.append(200);
  split->setSizes( vl );

//  connect( disk_lv, SIGNAL( Changed( QListViewItem * ) ),
//           this, SLOT( slotCatalogSelected( QListViewItem * ) ) );
  connect( lvD, SIGNAL( selectionChanged( QListViewItem * ) ),
           this, SLOT( slotRItemSelected( QListViewItem * ) ) );
  connect( lvM, SIGNAL( selectionChanged( QListViewItem * ) ),
           this, SLOT( slotLItemSelected( QListViewItem * ) ) );
  connect( ladd, SIGNAL( toggled( bool ) ),
           this, SLOT( slotToggledAdd( bool ) ) );
  connect( ledit, SIGNAL( toggled( bool ) ),
           this, SLOT( slotToggledLEdit( bool ) ) );
  connect( redit, SIGNAL( toggled( bool ) ),
           this, SLOT( slotToggledREdit( bool ) ) );
  connect( lmid->ldone, SIGNAL( clicked() ),
           this, SLOT( slotDone() ) );
  connect( rmid->ldone, SIGNAL( clicked() ),
           this, SLOT( slotDone() ) );
  connect( ldel, SIGNAL( clicked() ),
           this, SLOT( slotLDelete() ) );
  connect( rdel, SIGNAL( clicked() ),
           this, SLOT( slotRDelete() ) );
}

CatalogView::~CatalogView()
{
}

void CatalogView::setEntry( eKListViewItem *lv )
{
  CatalogEntry *ce = lv->treeItem->entry;

  lv->setText( 0, ce->Name() );

  if( lv->type )
    return;

  lv->setText( 1, QString::number( ce->Size() ) );
  lv->setText( 2, ( ce->Date() )->toString( "dd.MM.yyyy" ) );

  QStringList *l = ce->CatList();
  if( l != 0)
    lv->setText( 3, l->join( "::" ) );

  lv->setText( 4, ce->Description() );
  lv->setPixmap(
    0,
    KGlobal::iconLoader()->loadIcon( ce->Icon(), KIcon::Small ) );
}

void CatalogView::slotDone()
{
  // Prepare some variable
  QStringList *cl;

  QDate *cDate = new QDate( QDate::currentDate() );
  // Do the right thing
  switch( midlg )
  {
    case LAdd:
      ladd->setOn( false );
      lmid->hide();
      if( lmid->Name().isEmpty() )
        return;
      cl = lmid->CatList();
      if( !lmid->Mount().isEmpty() )
      {
        treeItemP = doc->logHand->newCatalog(
          lmid->Mount(),              // Folder where to start the scan
          lmid->Name(),               // Name of the catalog
          lmid->Description(),        // Description of the catalog
          cDate,                      // Date of creation of catalog
          cl,                         // List of categories
          lmid->isChecked() );        // Flag for the propagation of categories
      }
      else
      {
        treeItemP = doc->logHand->newCatalog(
          lmid->Name(),               // Name of the catalog
          lmid->Description(),        // Description of the catalog
          cDate,                      // Date of creation of catalog
          cl );                       // List of categories
      }
      lvMI = new eKListViewItem( lvM, treeItemP, true );
      lvM->setSelected( lvMI, true );
      setEntry( lvMI );
      addChildren( lvMI );
      break;
    case LEdit:
      ledit->setOn( false );
      if( lmid->Name().isEmpty() )
        return;
      cl = lmid->CatList();
      treeItemP->entry->SetName( lmid->Name() );
      treeItemP->entry->SetDescription( lmid->Description() );
      setCategories( treeItemP, cl, lmid->isChecked() ) ;
      setEntry( lvMI );
      lmid->hide();
      break;
    case REdit:
      redit->setOn( false );
      if( rmid->Name().isEmpty() )
        return;
      cl = rmid->CatList();
      treeItemC->entry->SetName( rmid->Name() );
      treeItemC->entry->SetDescription( rmid->Description() );
      setCategories( treeItemC, cl, lmid->isChecked() ) ;
      RefreshChildren( lvMI );
      rmid->hide();
      break;
  }
  RefreshRView();
  doc->setModified( true );
  emit( catalogChanged() );
}

/** No descriptions */
void CatalogView::slotToggledAdd( bool on )
{
  if( on )
  {
    midlg = LAdd;
    initMid();
    lmid->show();
  }
  else
    lmid->hide();
}

/** Show/Hide Edit Dialog */
void CatalogView::slotToggledLEdit( bool on )
{
  if( on )
  {
    if( treeItemP == 0 )
    {
      ledit->setOn( false );
      return;
    }
    midlg = LEdit;
    initMid();
    lmid->show();
  }
  else
    lmid->hide();
}

/** Show/Hide Edit Dialog */
void CatalogView::slotToggledREdit( bool on )
{
  if( on )
  {
    if( treeItemC == 0 )
    {
      redit->setOn( false );
      return;
    }
    midlg = REdit;
    initMid();
    rmid->show();
  }
  else
    rmid->hide();
}

/** Remove the selected entry */
void CatalogView::slotLDelete()
{
  if( !treeItemP )
    return;
  // Close left and right view
  ladd->setOn( false );
  ledit->setOn( false );
  redit->setOn( false );

  if( KMessageBox::warningYesNo( 0, i18n("Are you sure to remove the selected entry ?") ) ==
      KMessageBox::No )
    return;

  // delete the item ( in the right order !!! )
  if( treeItemP->entry->EntryType() == CatalogEntry::E_CATALOG )
    doc->logHand->removeCatalog( treeItemP );
  else
    delete treeItemP;
  delete lvMI;
  // initialize the pointers
  lvMI = 0;
  treeItemP = 0;
  // clear the view
  lvD->clear();
  // Notify the change
  emit( catalogChanged() );
  doc->setModified( true );
}

void CatalogView::slotLItemSelected( QListViewItem *lvi )
{
  lvMI = (eKListViewItem *)lvi;
  if( !lvi )
  {
    treeItemP = 0;
    return;
  }
  treeItemP = lvMI->treeItem;

  if( midlg == LAdd)
    initMid();
  RefreshRView();
}

void CatalogView::slotRItemSelected( QListViewItem *lvi )
{
  if( !lvi )
  {
    treeItemC = 0;
    return;
  }
  treeItemC = ( (eKListViewItem *)lvi )->treeItem;
}

void CatalogView::initMid()
{
  switch( midlg )
  {
    case LAdd:
      ledit->setOn( false );
      redit->setOn( false );
      lmid->setCaption( i18n("<B>Add a Catalog</B>") );
      lmid->setName( "" );
      lmid->setDescription( "" );
      lmid->MountEnabled( true );
      lmid->setCatList( doc->oryHand->categoriesList(), 0 );
      break;
    case LEdit:
      ladd->setOn( false );
      redit->setOn( false );
      lmid->setCaption( i18n("<B>Edit the Entry</B>") );
      lmid->setName( treeItemP->entry->Name() );
      lmid->setMount( "" );
      lmid->setDescription( treeItemP->entry->Description() );
      lmid->MountEnabled( false );
      lmid->setCatList( doc->oryHand->categoriesList(), treeItemP->entry->CatList() );
      break;
    case REdit:
      ladd->setOn( false );
      ledit->setOn( false );
      rmid->setCaption( i18n("<B>Edit the Entry</B>") );
      rmid->setName( treeItemC->entry->Name() );
      rmid->setDescription( treeItemC->entry->Description() );
      rmid->MountEnabled( false );
      rmid->setCatList( doc->oryHand->categoriesList(), treeItemC->entry->CatList() );
      break;
  }
}

/** No descriptions */
void CatalogView::addChildren( eKListViewItem *lvi )
{
  CatTreeItem *item = lvi->treeItem->child;
  while( item )
  {
    if( item->entry->EntryType() == CatalogEntry::E_DIR )
    {
      eKListViewItem *i = new eKListViewItem( lvi, item, true );
      setEntry( i );
      if( item->child )
        addChildren( i );
    }
    item = item->next;
  }
}

void CatalogView::RefreshRView()
{
  lvD->clear();
  if( !treeItemP )
    return;
  CatTreeItem *item = treeItemP->child;
  while( item )
  {
    eKListViewItem *i = new eKListViewItem( lvD, item, false );
    setEntry( i );
    item = item->next;
  }
  treeItemC = 0;
}

void CatalogView::setCategories( CatTreeItem *ti, QStringList *cl, bool propagate )
{
  ti->entry->SetCatList( cl );
  if( !propagate )
    return;

  CatTreeItem *item = ti->child;
  while( item )
  {
    item->entry->SetCatList( cl );
    if( item->child )
       setCategories( item->child, cl, true );
    item = item->next;
  }

}

void CatalogView::RefreshChildren( eKListViewItem *lvi )
{
  eKListViewItem* child = (eKListViewItem*)(lvi->firstChild());
  for( int c = 0; c < lvi->childCount(); c++ )
  {
    setEntry( child );
    child = (eKListViewItem*)(child->nextSibling());
  }
}

void CatalogView::slotRDelete()
{
  if( !treeItemC )
    return;
  // Close left and right view
  ladd->setOn( false );
  ledit->setOn( false );
  redit->setOn( false );

  if( KMessageBox::warningYesNo( 0, i18n("Are you sure to remove the selected entry ?") ) ==
      KMessageBox::No )
    return;

  // search in lvMI the element associated in Left View
  eKListViewItem* child = (eKListViewItem*)(lvMI->firstChild());
  eKListViewItem* item = 0;
  for( int c = 0; c < lvMI->childCount(); c++ )
  {
    if( child->treeItem == treeItemC )
    {
      item = child;
      break;
    }
    child = (eKListViewItem*)(child->nextSibling());
  }
  delete treeItemC;
  delete item;
  RefreshRView();
  doc->setModified( true );
}

void CatalogView::RefreshLView()
{
  lvM->clear();
  QValueListIterator<CatTreeItem *> it = doc->logHand->logList.begin();
  while ( it != doc->logHand->logList.end() )
  {
    lvMI = new eKListViewItem( lvM, *it, true );
    setEntry( lvMI );
    addChildren( lvMI );
    it++;
  }
  lvM->setSelected( lvM->firstChild(), true );
  RefreshRView();
  emit( catalogChanged() );
}
