/***************************************************************************
                          categoryview.cpp  -  description
                             -------------------
    begin                : Fri Mar 28 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "categoryview.h"
#include "categoryhandler.h"
#include "klistview.h"
#include "klocale.h"
#include "qlayout.h"
#include "qlineedit.h"
#include "qlabel.h"
#include "qpushbutton.h"

CategoryView::CategoryView( QWidget *parent, const char *name, CategoryHandler *ch ):
QWidget(parent,name)
{
  chandler = ch;

  // Main layout
  QVBoxLayout *vbox = new QVBoxLayout( this );

  // Top
  lv = new KListView( this );
  lv->addColumn( i18n( "Category" ) );
  lv->addColumn( i18n( "Description" ) );
  lv->setFullWidth();
  vbox->addWidget( lv );

  // Middle
  mid = new QFrame( this );
  mid->setFrameStyle( QFrame::Panel | QFrame::Raised );
  QVBoxLayout *midvb = new QVBoxLayout( mid, 6 );
  QHBoxLayout *midhbu = new QHBoxLayout( mid, 6 );
  QHBoxLayout *midhbd = new QHBoxLayout( mid, 6 );
  QLabel *lname = new QLabel( i18n("Name:"), mid );
  lcapt = new QLabel( i18n("<B>Add a Category</B>"), mid );

  ename = new QLineEdit( "", mid, "lineedit" );
  ename->setMinimumWidth(fontMetrics().maxWidth()*10);

  QLabel *ldesc = new QLabel( i18n("Description:"), mid );
  edesc = new QLineEdit( "", mid, "lineedit" );
  edesc->setMinimumWidth(fontMetrics().maxWidth()*20);
  QPushButton *done = new QPushButton( i18n("Done"), mid );
  midhbu->addWidget( lname );
  midhbu->addWidget( ename );
  midhbu->addStretch( 1 );
  midhbu->addWidget( ldesc );
  midhbu->addWidget( edesc );
  midhbu->addStretch( 1 );
  midhbd->addWidget( done );
  midhbd->addStretch( 1 );
  midvb->addWidget( lcapt );
  midvb->addLayout( midhbu );
  midvb->addLayout( midhbd );
  vbox->addWidget( mid );

  mid->hide();

  // Bottom
  QFrame *bot = new QFrame( this );
  QHBoxLayout *gl = new QHBoxLayout( bot, 10 );
  add = new QPushButton( i18n("&Add"), bot );
  edit = new QPushButton( i18n("E&dit"), bot );
  QPushButton *del = new QPushButton( i18n("&Remove"), bot );
  add->setFixedHeight( fontMetrics().height() + 2 );
  edit->setFixedHeight( fontMetrics().height() + 2 );
  del->setFixedHeight( fontMetrics().height() + 2 );
  gl->addSpacing( add->height() + 10 );
  add->setToggleButton( true );
  edit->setToggleButton( true );
  gl->addWidget( add );
  gl->addWidget( edit );
  gl->addWidget( del );
  gl->addStretch( 1 );
  vbox->addWidget( bot );

  connect( this, SIGNAL( categoryChanged() ), this, SLOT( slotInit() ) );
  connect( del, SIGNAL( clicked() ), this, SLOT( slotDelete() ) );
  connect( add, SIGNAL( toggled( bool ) ), this, SLOT( slotAdd( bool ) ) );
  connect( edit, SIGNAL( toggled( bool ) ), this, SLOT( slotEdit( bool ) ) );
  connect( done, SIGNAL( clicked() ), this, SLOT( slotDone() ) );
  connect( lv, SIGNAL( selectionChanged( QListViewItem * ) ),
           this, SLOT( slotSelected( QListViewItem * ) ) );
  slotInit();
}

CategoryView::~CategoryView()
{
}

/** Used to delete a category */
void CategoryView::slotDelete()
{
  ca = 0;
  if( add->state() )
    add->setOn( false );
  if( edit->state() )
    edit->setOn( false );

  QListViewItem * item = lv->selectedItem();

  if( item )
  {
    chandler->delCategory( item->text( 0 ) );
    emit( categoryChanged() );
    emit( categoryDeleted( item->text( 0 ) ) );
  }
}

/** Used to initialize the view */
void CategoryView::slotInit()
{
  if( add->state() )
    add->setOn( false );
  if( edit->state() )
    edit->setOn( false );
  CategoryHandler::iterator it = chandler->begin();
  lv->clear();
  while( it != chandler->end() )
  {
    new KListViewItem(lv, (*it).Name(), (*it).Description() );
    it++;
  }
  lv->setSelected( lv->firstChild(), TRUE );
}

/** Used to add an item to category */
void CategoryView::slotAdd( bool on )
{
  if( on )
  {
    ca = 0;
    initMid();
    mid->show();
  }
  else
    mid->hide();
}

/** Used to modify a category */
void CategoryView::slotEdit( bool on )
{
  if( on )
  {
    QListViewItem *item = lv->selectedItem();
    if( !item )
    {
      if( edit->state() )
        edit->setOn( false );
      return;
    }

    ca = chandler->find( item->text( 0 ) );
    if( !ca )
    {
      if( edit->state() )
        edit->setOn( false );
      return;
    }

    initMid();
    mid->show();
  }
  else
    mid->hide();
}

/** No descriptions */
void CategoryView::slotDone()
{
  if( ename->text().isEmpty() )
    return;
  if( !ca )
  {                        // Add
    chandler->addCategory( ename->text(), edesc->text() );
    if( add->state() )
      add->setOn( false );
  }
  else
  {                        // Edit
    ca->setName( ename->text() );
    ca->setDescription( edesc->text() );
    if( edit->state() )
      edit->setOn( false );
  }
  emit( categoryChanged() );
}

/** No descriptions */
void CategoryView::slotSelected( QListViewItem *it )
{
  if( it )
  {
    ca = chandler->find( it->text( 0 ) );
    ename->setText( ca->Name() );
    edesc->setText( ca->Description() );
  }
}

/** Initialize the mid view */
void CategoryView::initMid()
{
  if( ca )
  {
    if( add->state() )
      add->setOn( false );
    lcapt->setText( "<B>Edit the category</B>" );
    ename->setText( ca->Name() );
    edesc->setText( ca->Description() );
  }
  else
  {
    if( edit->state() )
      edit->setOn( false );
    lcapt->setText( "<B>Add a Category</B>" );
    ename->setText( "" );
    edesc->setText( "" );
  }
}
