/***************************************************************************
                          cattree.cpp  -  description
                             -------------------
    begin                : Tue Apr 15 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cattreeitem.h"
#include "catalogentry.h"

CatTreeItem::CatTreeItem( CatTreeItem *p, CatalogEntry *ce )
{
  parent = p;
  entry = ce;
  child = 0;
  next = 0;
  numChild = 0;
  if( parent )
  {
    next = parent->child;
    parent->child = this;
    parent->numChild++;
  }

  if( parent )
    entry->setMyCatalog( parent->entry->MyCatalog() );
  else
    entry->setMyCatalog( ce->Name() );
}

CatTreeItem::~CatTreeItem()
{
  // 1 -  isolate the branch from tree
  if( parent )
  {
    // 1.1 - if this is the first child:
    if( parent->child == this )
      parent->child = next;
    // 1.2 - else search the previous element and close the links:
    else
    {
      CatTreeItem *item = parent->child;
      while( item->next != this )
        item = item->next;
      item->next = next; // Previous now corresponds to Next
    }
  }

  // 2 - Free the contents of item
  delete entry;

  // 3 - Delete each child
  CatTreeItem *item = child;
  while( item )
  {
    delete item;
    item = child; // "child" now is "next" because delete change pointers
  }
}
