/***************************************************************************
                          midview.cpp  -  description
                             -------------------
    begin                : Fri Apr 11 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qpainter.h>
#include <qsplitter.h>
#include <qdatetime.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qstringlist.h>

// include files for KDE
#include <klocale.h>
#include <klistview.h>
#include <kmessagebox.h>
#include <kmimetype.h>
#include <kiconloader.h>
#include <kfiledialog.h>

#include "midview.h"

MidView::MidView( QWidget *parent ):
QFrame( parent )
{
  setFrameStyle( QFrame::Panel | QFrame::Raised );
  QGridLayout *lmgl = new QGridLayout( this, 8, 3, 6 );

  lcapt = new QLabel( "", this );
  lmgl->addMultiCellWidget( lcapt, 0, 0, 0, 2 );

  ename = new QLineEdit( "", this, "name" );
  ename->setMinimumWidth(fontMetrics().maxWidth()*20);
  QLabel *lname = new QLabel( i18n("Catalog Name:"), this );
  lmgl->addWidget( lname, 1, 0 );
  lmgl->addMultiCellWidget( ename, 1, 1, 1, 2 );

  edesc = new QLineEdit( "", this, "mount" );
  edesc->setMinimumWidth(fontMetrics().maxWidth()*20);
  QLabel *ldesc = new QLabel( i18n("Description:"), this );
  lmgl->addWidget( ldesc, 2, 0 );
  lmgl->addMultiCellWidget( edesc, 2, 2, 1, 2 );

  lmount = new QLabel( i18n("Mount Point:"), this );
  emount = new QLineEdit( "", this, "mount" );
  emount->setMinimumWidth(fontMetrics().maxWidth()*20);
  lchoose = new QPushButton( "", this );
  lchoose->setPixmap( KGlobal::iconLoader()->loadIcon( "fileopen", KIcon::Small ) );
  lmgl->addWidget( lmount, 3, 0 );
  lmgl->addWidget( emount, 3, 1 );
  lmgl->addWidget( lchoose, 3, 2 );

  eCategory = new QListBox( this );
  eCategory->setMaximumHeight( 60 );
  eCategory->setSelectionMode( QListBox::Multi );
  QLabel* lCategory = new QLabel( i18n("Category:"), this );
  lmgl->addWidget( lCategory, 4, 0 );
  lmgl->addMultiCellWidget( eCategory, 4, 5, 1, 2 );

  lmgl->addRowSpacing( 5, 40 );

  check = new QCheckBox( i18n("Propagate categories to children"), this );
  check->setChecked( true );
  lmgl->addWidget( check, 6, 1 );

  ldone = new QPushButton( i18n("Done"), this );

  lmgl->addWidget( ldone, 7, 0 );

  connect( lchoose, SIGNAL( clicked() ), this, SLOT( slotOpen() ) );
}

MidView::~MidView()
{
}

/** Used to choose the mount point */
void MidView::slotOpen()
{
  emount->setText(
    KFileDialog::getExistingDirectory( "/", this, "Select the mount point" ) );
}

/** Returns the choosed categories */
QStringList *MidView::CatList()
{
  QStringList *selectList = new QStringList();
  int c = eCategory->count();
  int s = 0;
  for( int i = 0; i < c; i++ )
  {
    if( eCategory->isSelected( i ) )
    {
      selectList->append( eCategory->text( i ) );
      s++;
    }
  }
  if( s )
    return selectList;
  else
    return 0;
}

/** sets the categories in the listbox */
void MidView::setCatList( QStringList *cl, QStringList *itemcl )
{
  eCategory->clear();

  if( !cl )
    return;

  eCategory->insertStringList( *cl );

  if( !itemcl )
    return;

  int c1 = itemcl->size();
  int c2 = eCategory->count();

  for( int j = 0;  j < c2; j++ )
  {
    for( int i = 0; i < c1; i++ )
    {
      if( (*cl)[i] == eCategory->text( j ) )
        eCategory->setSelected( j, true );
      else
        eCategory->setSelected( j, false );
    }
  }
}

/** Set the name of the entry or the catalogue */
void MidView::setName( QString str )
{
  ename->setText( str );
}

/** Returns the name of the entry or the catalogue */
QString MidView::Name()
{
  return ename->text();
}

/** Set the name of the entry or the catalogue */
void MidView::setMount( QString str )
{
  emount->setText( str );
}

/** Returns the name of the entry or the catalogue */
QString MidView::Mount()
{
  return emount->text();
}

/** Set the name of the entry or the catalogue */
void MidView::setDescription( QString str )
{
  edesc->setText( str );
}

/** Returns the description of the entry or the catalogue */
QString MidView::Description()
{
  return edesc->text();
}

/** Show/Hide the mount point dialog */
void MidView::MountEnabled( bool on )
{
  if( on )
  {
    lmount->show( );
    emount->show( );
    lchoose->show( );
  }
  else
  {
    lmount->hide( );
    emount->hide( );
    lchoose->hide( );
  }

}

/** Set the caption of the dialog */
void MidView::setCaption( QString str )
{
  lcapt->setText( str );
}

/** Return the state of the check box */
bool MidView::isChecked()
{
  return check->isChecked();
}
